# Student auth
# Getters about private information
from .attendance import getAttendanceRecords
from .auth import LoginCreds, Student, getStudent
from .marks import getMarkHistory
from .payment import getPaymentInfo
from .timetable import getTimetable
